@@ -12,9 +12,17 @@ GERAPY_PYPPETEER_WINDOW_HEIGHT = 700
 
 # pyppeteer settings
 GERAPY_PYPPETEER_HEADLESS = True
+GERAPY_PYPPETEER_EXECUTABLE_PATH = None
+GERAPY_PYPPETEER_IGNORE_HTTPS_ERRORS = False
+GERAPY_PYPPETEER_SLOW_MO = None
+GERAPY_PYPPETEER_IGNORE_DEFAULT_ARGS = False
+GERAPY_PYPPETEER_HANDLE_SIGINT = True
+GERAPY_PYPPETEER_HANDLE_SIGTERM = True
+GERAPY_PYPPETEER_HANDLE_SIGHUP = True
 GERAPY_PYPPETEER_DUMPIO = False
 GERAPY_PYPPETEER_DEVTOOLS = False
-GERAPY_PYPPETEER_EXECUTABLE_PATH = None
+GERAPY_PYPPETEER_AUTO_CLOSE = True
+# pyppeteer args
 GERAPY_PYPPETEER_DISABLE_EXTENSIONS = True
 GERAPY_PYPPETEER_HIDE_SCROLLBARS = True
 GERAPY_PYPPETEER_MUTE_AUDIO = True
